package core

import (
	"git.akyoto.dev/cli/q/src/cpu"
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/expression"
	"git.akyoto.dev/cli/q/src/token"
	"git.akyoto.dev/cli/q/src/types"
)

// BeforeCall loads the registers with the parameter values and checks that the types match with the function signature.
func (f *Function) BeforeCall(fn *Function, parameters []*expression.Expression, registers []cpu.Register) error {
	for i := len(parameters) - 1; i >= 0; i-- {
		typ, err := f.ExpressionToRegister(parameters[i], registers[i])

		if err != nil {
			return err
		}

		if !types.Is(typ, fn.Input[i].Type) {
			_, expectsPointer := fn.Input[i].Type.(*types.Pointer)

			if expectsPointer && parameters[i].Token.Kind == token.Number && parameters[i].Token.Text(f.File.Bytes) == "0" {
				continue
			}

			return errors.New(&errors.TypeMismatch{Encountered: typ.Name(), Expected: fn.Input[i].Type.Name(), ParameterName: fn.Input[i].Name}, f.File, parameters[i].Token.Position)
		}
	}

	return nil
}
